//+------------------------------------------------------------------+
//|                                            Coppock curve CCT.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_separate_window
#property indicator_buffers  4
#property indicator_color1   Red
#property indicator_color2   Green
#property indicator_color3   Red
#property indicator_color4   DimGray
#property indicator_width4   2

//
//
//
//
//

extern int     ROCPeriod1     = 14;
extern int     ROCPeriod2     = 11;
extern int     Signal         =  5;
extern bool    ShowHistogram  = true;
extern string  timeFrame      = "Current time frame";

//
//
//
//
//

double   Buffer[];
double   HistBufferU[];
double   HistBufferD[];
double   SignalBuffer[];
datetime TimeArray[];
int      TimeFrame;
int      CopyLabel;
string   ShortName;
string   IndicatorFileName;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int init()
{
   SetIndexBuffer(0,Buffer);
   SetIndexLabel(0,"Coppock curve");
   SetIndexBuffer(3,SignalBuffer);
   if (Signal > 0)
         SetIndexLabel(3,"Coppock curve signal");
   else  SetIndexLabel(3,NULL);

   //
   //
   //
   //
   //
   
   if (ShowHistogram)
      {
         SetIndexBuffer(1,HistBufferU);
         SetIndexBuffer(2,HistBufferD);
         SetIndexStyle(1,DRAW_HISTOGRAM);
         SetIndexStyle(2,DRAW_HISTOGRAM);
         SetIndexLabel(1,"Coppock curve");
         SetIndexLabel(2,"Coppock curve");
      }
   else
      {
         SetIndexStyle(1,DRAW_NONE);
         SetIndexStyle(2,DRAW_NONE);
         SetIndexLabel(1,NULL);
         SetIndexLabel(2,NULL);
      }

   //
   //
   //
   //
   //

   TimeFrame = stringToTimeFrame(timeFrame);
   ShortName = "Coppock curve CCT "+TimeFrameToString(TimeFrame)+" ("+ROCPeriod1+","+ROCPeriod2+","+Signal+")";
   IndicatorShortName(ShortName);
   IndicatorFileName = WindowExpertName();

   //
   //
   //
   //
   //

   return(0);
}
int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   int    counted_bars=IndicatorCounted();
   int    limit,i,k;
   double sum;


   if(counted_bars < 0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = Bars-counted_bars;


   //
   //
   //
   //
   //
   
   if (TimeFrame != Period())
      {
         limit = MathMax(limit,TimeFrame/Period());
         ArrayCopySeries(TimeArray ,MODE_TIME ,NULL,TimeFrame);
         
         //
         //
         //
         //
         //
         
         for(i=0,int y=0; i<limit; i++)
         {
            if(Time[i]<TimeArray[y]) y++;
            Buffer[i]       = iCustom(NULL,TimeFrame,IndicatorFileName,ROCPeriod1,ROCPeriod2,Signal,ShowHistogram,0,y);
            HistBufferU[i]  = iCustom(NULL,TimeFrame,IndicatorFileName,ROCPeriod1,ROCPeriod2,Signal,ShowHistogram,1,y);
            HistBufferD[i]  = iCustom(NULL,TimeFrame,IndicatorFileName,ROCPeriod1,ROCPeriod2,Signal,ShowHistogram,2,y);
            SignalBuffer[i] = iCustom(NULL,TimeFrame,IndicatorFileName,ROCPeriod1,ROCPeriod2,Signal,ShowHistogram,3,y);
         }
         return(0);         
      }
         
   //
   //
   //
   //
   //

   for (i=limit;i>=0;i--)
      {
         for (sum=0.00,k=0;k<10;k++) sum += iROC(ROCPeriod1,i+k)*(10-k)+iROC(ROCPeriod2,i+k)*(10-k);

            //
            //
            //
            //
            //
                    
            Buffer[i] = sum/2;
            if (ShowHistogram)
               {
                  HistBufferU[i] = HistBufferU[i+1];
                  HistBufferD[i] = HistBufferD[i+1];
                     if (Buffer[i] > Buffer[i+1]) { HistBufferU[i] = Buffer[i];  HistBufferD[i] = EMPTY_VALUE; }
                     if (Buffer[i] < Buffer[i+1]) { HistBufferD[i] = Buffer[i];  HistBufferU[i] = EMPTY_VALUE; }
               }                  
      }
   for (i=limit;i>=0;i--) SignalBuffer[i] = iMAOnArray(Buffer,0,Signal,0,MODE_SMA,i);
   
   //
   //
   //
   //
   //
   
   return(0);
}
 

//+------------------------------------------------------------------+
//|
//+------------------------------------------------------------------+
//
//
//
//
//

double iROC(int period, int shift)
{
   double ROC;

   if (Close[shift + period] != 0)
          ROC = (Close[shift]-Close[shift + period]) / Close[shift + period];
   else   ROC = 0.00;
   return(ROC*100.00);
}

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   int tf=0;
       tfs = StringTrimLeft(StringTrimRight(StringUpperCase(tfs)));
         if (tfs=="M1" || tfs=="1")     tf=PERIOD_M1;
         if (tfs=="M5" || tfs=="5")     tf=PERIOD_M5;
         if (tfs=="M15"|| tfs=="15")    tf=PERIOD_M15;
         if (tfs=="M30"|| tfs=="30")    tf=PERIOD_M30;
         if (tfs=="H1" || tfs=="60")    tf=PERIOD_H1;
         if (tfs=="H4" || tfs=="240")   tf=PERIOD_H4;
         if (tfs=="D1" || tfs=="1440")  tf=PERIOD_D1;
         if (tfs=="W1" || tfs=="10080") tf=PERIOD_W1;
         if (tfs=="MN" || tfs=="43200") tf=PERIOD_MN1;
         if (tf<Period()) tf=Period();
  return(tf);
}
string TimeFrameToString(int tf)
{
   string tfs="";
   
   if (tf!=Period())
      switch(tf) {
         case PERIOD_M1:  tfs="M1"  ; break;
         case PERIOD_M5:  tfs="M5"  ; break;
         case PERIOD_M15: tfs="M15" ; break;
         case PERIOD_M30: tfs="M30" ; break;
         case PERIOD_H1:  tfs="H1"  ; break;
         case PERIOD_H4:  tfs="H4"  ; break;
         case PERIOD_D1:  tfs="D1"  ; break;
         case PERIOD_W1:  tfs="W1"  ; break;
         case PERIOD_MN1: tfs="MN1";
      }
   return(tfs);
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;
   int      lenght = StringLen(str) - 1;
   int      char;
   
   while(lenght >= 0)
      {
         char = StringGetChar(s, lenght);
         
         //
         //
         //
         //
         //
         
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                  s = StringSetChar(s, lenght, char - 32);
         else 
              if(char > -33 && char < 0)
                  s = StringSetChar(s, lenght, char + 224);
         lenght--;
   }
   
   //
   //
   //
   //
   //
   
   return(s);
}